@@ -18,6 +18,7 @@ import logging
 import asyncio
 import contextvars
 import inspect
+import warnings
 from collections import deque
 from functools import partial, reduce
 import copy
@@ -116,7 +117,7 @@ class AsyncTransition(Transition):
 
         machine = event_data.machine
         # cancel running tasks since the transition will happen
-        await machine.switch_model_context(event_data.model)
+        await machine.cancel_running_transitions(event_data.model, event_data.event.name)
 
         await event_data.machine.callbacks(event_data.machine.before_state_change, event_data)
         await event_data.machine.callbacks(self.before, event_data)
@@ -189,7 +190,8 @@ class AsyncEvent(Event):
             if self._is_valid_source(event_data.state):
                 await self._process(event_data)
         except BaseException as err:  # pylint: disable=broad-except; Exception will be handled elsewhere
-            _LOGGER.error("%sException was raised while processing the trigger: %s", self.machine.name, err)
+            _LOGGER.error("%sException was raised while processing the trigger '%s': %s",
+                          self.machine.name, event_data.event.name, repr(err))
             event_data.error = err
             if self.machine.on_exception:
                 await self.machine.callbacks(self.machine.on_exception, event_data)
@@ -305,18 +307,21 @@ class AsyncMachine(Machine):
                  send_event=False, auto_transitions=True,
                  ordered_transitions=False, ignore_invalid_triggers=None,
                  before_state_change=None, after_state_change=None, name=None,
-                 queued=False, prepare_event=None, finalize_event=None, model_attribute='state', on_exception=None,
-                 **kwargs):
-        self._transition_queue_dict = {}
-        super().__init__(model=model, states=states, initial=initial, transitions=transitions,
+                 queued=False, prepare_event=None, finalize_event=None, model_attribute='state',
+                 model_override=False, on_exception=None, on_final=None, **kwargs):
+
+        super().__init__(model=None, states=states, initial=initial, transitions=transitions,
                          send_event=send_event, auto_transitions=auto_transitions,
                          ordered_transitions=ordered_transitions, ignore_invalid_triggers=ignore_invalid_triggers,
                          before_state_change=before_state_change, after_state_change=after_state_change, name=name,
-                         queued=queued, prepare_event=prepare_event, finalize_event=finalize_event,
-                         model_attribute=model_attribute, on_exception=on_exception, **kwargs)
-        if self.has_queue is True:
-            # _DictionaryMock sets and returns ONE internal value and ignores the passed key
-            self._transition_queue_dict = _DictionaryMock(self._transition_queue)
+                         queued=bool(queued), prepare_event=prepare_event, finalize_event=finalize_event,
+                         model_attribute=model_attribute, model_override=model_override,
+                         on_exception=on_exception, on_final=on_final, **kwargs)
+
+        self._transition_queue_dict = _DictionaryMock(self._transition_queue) if queued is True else {}
+        self._queued = queued
+        for model in listify(model):
+            self.add_model(model)
 
     def add_model(self, model, initial=None):
         super().add_model(model, initial)
@@ -371,18 +376,24 @@ class AsyncMachine(Machine):
         return await asyncio.gather(*[func() for func in callables])
 
     async def switch_model_context(self, model):
+        warnings.warn("Please replace 'AsyncMachine.switch_model_context' with "
+                      "'AsyncMachine.cancel_running_transitions'.", category=DeprecationWarning)
+        await self.cancel_running_transitions(model)
+
+    async def cancel_running_transitions(self, model, msg=None):
         """
         This method is called by an `AsyncTransition` when all conditional tests have passed
         and the transition will happen. This requires already running tasks to be cancelled.
         Args:
             model (object): The currently processed model
+            msg (str): Optional message to pass to a running task's cancel request
         """
         for running_task in self.async_tasks.get(id(model), []):
             if self.current_context.get() == running_task or running_task in self.protected_tasks:
                 continue
             if running_task.done() is False:
                 _LOGGER.debug("Cancel running tasks...")
-                running_task.cancel()
+                running_task.cancel(msg)
 
     async def process_context(self, func, model):
         """
@@ -396,7 +407,7 @@ class AsyncMachine(Machine):
             bool: returns the success state of the triggered event
         """
         if self.current_context.get() is None:
-            self.current_context.set(asyncio.current_task())
+            token = self.current_context.set(asyncio.current_task())
             if id(model) in self.async_tasks:
                 self.async_tasks[id(model)].append(asyncio.current_task())
             else:
@@ -407,6 +418,7 @@ class AsyncMachine(Machine):
                 res = False
             finally:
                 self.async_tasks[id(model)].remove(asyncio.current_task())
+                self.current_context.reset(token)
                 if len(self.async_tasks[id(model)]) == 0:
                     del self.async_tasks[id(model)]
         else:
@@ -674,12 +686,30 @@ class AsyncTimeout(AsyncState):
                 await asyncio.shield(self._process_timeout(event_data))
             except asyncio.CancelledError:
                 pass
-
-        return asyncio.ensure_future(_timeout())
+        return asyncio.create_task(_timeout())
 
     async def _process_timeout(self, event_data):
         _LOGGER.debug("%sTimeout state %s. Processing callbacks...", event_data.machine.name, self.name)
-        await event_data.machine.callbacks(self.on_timeout, event_data)
+        event_data = AsyncEventData(event_data.state, AsyncEvent("timeout", event_data.machine),
+                                    event_data.machine, event_data.model, args=tuple(), kwargs={})
+        token = AsyncMachine.current_context.set(None)
+        try:
+            await event_data.machine.callbacks(self.on_timeout, event_data)
+        except BaseException as err:
+            _LOGGER.warning("%sException raised while processing timeout!",
+                            event_data.machine.name)
+            event_data.error = err
+            try:
+                if event_data.machine.on_exception:
+                    await event_data.machine.callbacks(event_data.machine.on_exception, event_data)
+                else:
+                    raise
+            except BaseException as err2:
+                _LOGGER.error("%sHandling timeout exception '%s' caused another exception: %s. "
+                              "Cancel running transitions...", event_data.machine.name, repr(err), repr(err2))
+                await event_data.machine.cancel_running_transitions(event_data.model, "timeout")
+        finally:
+            AsyncMachine.current_context.reset(token)
         _LOGGER.info("%sTimeout state %s processed.", event_data.machine.name, self.name)
 
     @property
